from sherbystats.anova import *
from sherbystats.doe import *
from sherbystats.mlr import *
from sherbystats.lda import *
from sherbystats.normplot import *
from sherbystats.pca import *
from sherbystats.pca_trou import *
from sherbystats.xlsread import *

from sherbystats.ternaire import *
from sherbystats.binaire import *
from sherbystats.binaire3D import *










